#!/usr/bin/env perl

use strict;
use warnings;

use Test::More;

plan tests => 72;

use_ok('HTML::Form::Simple');

my $form = HTML::Form::Simple->new;
ok($form, 'new form');

# form

is(
	$form->start,
	'<form>',
	'empty start'
);

is(
	$form->start({method => 'get', action => '', ignore => undef}),
	'<form action="" method="get">',
	'start with attributes'
);

is(
	eval { $form->start('should be a hashref') },
	undef,
	'invalid attributes'
);

is(
	$form->stop,
	'</form>',
	'stop'
);

# hidden

is(
	$form->hidden,
	'<input type="hidden">',
	'empty hidden'
);

is(
	$form->hidden(foo => 'bar'),
	'<input name="foo" type="hidden" value="bar">',
	'hidden'
);

is(
	$form->hidden(undef, undef, {value => 'bar', name => 'foo', id => 'foo'}),
	'<input id="foo" name="foo" type="hidden" value="bar">',
	'hidden options'
);

is_deeply(
	[ $form->hidden(foo => [1, 0], {class => 'test'}) ],
	[
		'<input class="test" name="foo" type="hidden" value="1">',
		'<input class="test" name="foo" type="hidden" value="0">',
	],
	'hidden array'
);

is_deeply(
	[ $form->hidden({2 => 0, 10 => 1}, {class => 'test'}) ],
	[
		'<input class="test" name="10" type="hidden" value="1">',
		'<input class="test" name="2" type="hidden" value="0">',
	],
	'hidden hash'
);

{
	local $, = "\n";
	is(
		scalar $form->hidden({2 => [1, 0], 10 => 2}, {class => 'test'}),
		join("\n",
			'<input class="test" name="10" type="hidden" value="2">',
			'<input class="test" name="2" type="hidden" value="1">',
			'<input class="test" name="2" type="hidden" value="0">',
		),
		'scalar hiddens'
	);
}

# submit

is(
	$form->submit,
	'<input type="submit">',
	'empty submit'
);

is(
	$form->submit('<OK>'),
	'<input type="submit" value="&lt;OK>">',
	'submit value'
);

is(
	$form->submit({disabled => 1, id => 'test', type => 'button'}),
	'<input disabled id="test" type="button">',
	'submit attributes'
);

is(
	$form->submit('<OK>', {type => '', value => 'override', id => ''}),
	'<input value="&lt;OK>">',
	'submit overrides'
);

# input

is(
	$form->text,
	'<input type="text">',
	'empty input'
);

is(
	$form->text(undef, undef, undef),
	'<input type="text">',
	'explicit empty input'
);

is(
	$form->text('test'),
	'<input id="test" name="test" type="text">',
	'input with name'
);

is(
	$form->text(undef, 'test'),
	'<input type="text" value="test">',
	'input with value'
);

is(
	$form->text(undef, {value => 'test'}),
	'<input type="text" value="test">',
	'input with attribute value'
);

is(
	$form->text({name => 'test', value => ''}),
	'<input id="test" name="test" type="text" value="">',
	'input with only attributes'
);

is(
	$form->text('', '', {
		disabled => 0,
		something => undef,
		class => undef,
		style => '',
		name => 'ignore',
		value => 'overrides',
	}),
	'<input name="" type="text" value="">',
	'input with empty attributes'
);

is(
	$form->text(undef, undef, {name => '0', value => '0'}),
	'<input id="0" name="0" type="text" value="0">',
	'input overrides'
);

is(
	$form->text('name', {id => ''}),
	'<input name="name" type="text">',
	'input with id override'
);

is(
	$form->text('<">', '<">', {id => '>"<'}),
	'<input id=">&quot;&lt;" name="&lt;&quot;>" type="text" value="&lt;&quot;>">',
	'input quoting'
);

is(
	$form->text(undef, {disabled => 'something'}),
	'<input disabled type="text">',
	'disabled input'
);

is(
	$form->text({type => 'password'}),
	'<input type="password">',
	'password'
);

# textarea

is(
	$form->text({rows => 0}),
	'<textarea rows="0"></textarea>',
	'minimal textarea'
);

is(
	$form->text(foo => 'bar', {cols => 42, rows => 1, disabled => 1}),
	'<textarea disabled cols="42" id="foo" name="foo" rows="1">bar</textarea>',
	'textarea'
);

is(
	$form->text(undef, qq{<foo>&bl'a"\n    .}, {cols => undef, rows => '<">'}),
	qq{<textarea rows="&lt;&quot;>">&lt;foo>&amp;bl'a&quot;\n    .</textarea>},
	'textarea quoting'
);

# select

is_deeply(
	[ $form->select ],
	[ qw(<select> </select>) ], # malformed html: at least 1 option required
	'empty select'
);

is_deeply(
	[ $form->select(undef, [], '', {name => '', class => ''}) ],
	[ qw(<select> </select>) ],
	'explicit empty select'
);

is_deeply(
	[ $form->select(undef, [undef]) ],
	[ qw(<select> <option> </select>) ],
	'minimal select'
);

is_deeply(
	[ $form->select(foo => [1, 2]) ],
	[
		'<select id="foo" name="foo">',
		'<option value="1">', '<option value="2">',
		'</select>'
	],
	'select contents'
);

is_deeply(
	[ $form->select(foo => [1, 2], 3) ],
	[
		'<select id="foo" name="foo">',
		'<option value="1">', '<option value="2">',
		'</select>'
	],
	'select invalid default'
);

is_deeply(
	[ $form->select(undef, [1, 2], 2) ],
	[
		'<select>',
		'<option value="1">', '<option selected value="2">',
		'</select>'
	],
	'select default'
);

is_deeply(
	[
		$form->select(foo => [
			'<">', {value => 2, disabled => 1, selected => 0, class => 1}, {selected => 1}
		], '2', {id => '', class => 'a <b', value => 1})
	],
	[
		'<select class="a &lt;b" name="foo">',
		'<option value="&lt;&quot;>">',
		'<option selected disabled class="1" value="2">',
		'<option selected>',
		'</select>'
	],
	'complex select'
);

is(
	scalar $form->select(foo => [1, 2]),
	'<select id="foo" name="foo"><option value="1"><option value="2"></select>',
	'select scalar'
);

# radio

is_deeply(
	[ $form->select(foo => [1], {type => 'radio'}) ],
	[ '<input id="foo_1" name="foo" type="radio" value="1">' ],
	'input select'
);

is_deeply(
	[
		$form->select(foo => [
			1, {value => 2, name => '', label => ''}, {value => 3, id => '', type => ''}
		], {type => 'checkbox', label => {3 => 'test', 2 => 'ignore'}, value => '1'})
	],
	[
		'<input checked id="foo_1" name="foo" type="checkbox" value="1">',
		'<input id="foo_2" name="" type="checkbox" value="2">',
		'<label><input name="foo" value="3"> test</label>',
	],
	'input selects'
);

is(
	$form->radio(foo => 'test'),
	'<input id="foo_test" name="foo" type="radio" value="test">',
	'radio method'
);

is(
	$form->radio(foo => undef, 'test'),
	'<label><input id="foo_1" name="foo" type="radio" value="1"> test</label>',
	'labeled radio'
);

is(
	$form->radio(undef, {checked => 1}),
	'<input checked type="radio">',
	'simple radio button'
);

is_deeply(
	[ $form->radio(undef, ['', '<">']) ],
	[
		'<input type="radio" value="">',
		'<input type="radio" value="&lt;&quot;>">',
	],
	'multiple radios'
);

is_deeply(
	[ $form->radio(undef, undef, ['', '<">']) ],
	[
		'<input type="radio" value="1">',
		'<label><input type="radio" value="2"> <"></label>',
	],
	'multiple radio labels'
);

is_deeply(
	[ $form->radio(undef, [0, 1], undef, 0) ],
	[
		'<input checked type="radio" value="0">',
		'<input type="radio" value="1">',
	],
	'radio default'
);

is_deeply(
	[ $form->radio(foo => ['foo', ''], 'test', {value => '', id => ''}) ],
	[
		'<label><input name="foo" type="radio" value="foo"> test</label>',
		'<label><input checked name="foo" type="radio" value=""> test</label>',
	],
	'multiple radios with custom value'
);

is_deeply(
	[ $form->radio(foo => [0, 1, ''], ['', 0]) ],
	[
		'<input id="foo_0" name="foo" type="radio" value="0">',
		'<label><input id="foo_1" name="foo" type="radio" value="1"> 0</label>',
		'<input id="foo_" name="foo" type="radio" value="">',
	],
	'multiple radios with custom values'
);

{
	# make sure arguments aren't modified
	my @args = (foo => [0, {value => 1}], [0, 1], {name => 0, value => 1});
	my @orgs = (foo => [0, {value => 1}], [0, 1], {name => 0, value => 1});
	my @output = (
		'<label><input id="foo_0" name="foo" type="radio" value="0"> 0</label>',
		'<label><input checked id="foo_1" name="foo" type="radio" value="1"> 1</label>',
	);

	is_deeply(
		[ $form->radio(@args) ],
		\@output,
		'options var to radio'
	);

	is_deeply(
		[ $form->check(@args) ],
		[
			'<label><input checked id="foo_1" name="foo" type="checkbox" value="1"> 0</label>',
			'<input checked id="foo_1" name="foo" type="checkbox" value="1">',
		],
		'options var to check'
	);

	is(
		scalar $form->radio(@args),
		join('', @output),
		'options var again to radio'
	);

	is_deeply(\@args, \@orgs, 'options var unmodified');
}

# check

is(
	$form->check('foo'),
	'<input id="foo" name="foo" type="checkbox" value="1">',
	'check method'
);

is(
	$form->check(foo => '<">'),
	'<label><input id="foo" name="foo" type="checkbox" value="1"> <"></label>',
	'labeled checkbox'
);

is(
	$form->check(foo => {label => 'test', value => undef}, {disabled => 1}),
	'<label><input disabled id="foo" name="foo" type="checkbox"> test</label>',
	'check parameters'
);

is_deeply(
	[ $form->check(undef, '', 1) ],
	[ '<input checked type="checkbox" value="1">' ],
	'anonymous checkbox'
);

is(
	$form->check(foo => [undef]),
	'<input id="foo_1" name="foo" type="checkbox" value="1">',
	'multipart check'
);

is_deeply(
	[ $form->check(foo => [{value => undef}, undef]) ],
	[
		'<input id="foo" name="foo" type="checkbox">',
		'<input id="foo_2" name="foo" type="checkbox" value="2">',
	],
	'multiple checkboxes'
);

is(
	$form->check(foo => [undef], {id => ''}),
	'<input name="foo" type="checkbox" value="1">',
	'idless checks'
);

is_deeply(
	[ $form->check(
		foo => [ {id => 'quux'}, {name => 'name'}, {value => 0}, {id => ''}, undef ], {id => 'bar'}
	) ],
	[
		'<input id="quux" name="foo" type="checkbox" value="1">',
		'<input id="bar_2" name="name" type="checkbox" value="2">',
		'<input id="bar_0" name="foo" type="checkbox" value="0">',
		'<input name="foo" type="checkbox" value="4">',
		'<input id="bar_5" name="foo" type="checkbox" value="5">',
	],
	'check overrides'
);

is_deeply(
	[ $form->check(foo => ['bar', {name => 'bar'}], {name => 'ignored'}) ],
	[
		'<label><input id="foo_1" name="foo" type="checkbox" value="1"> bar</label>',
		'<input id="bar_2" name="bar" type="checkbox" value="2">',
	],
	'checkbox names'
);

is_deeply(
	[ $form->check(undef, ['', '<">']) ],
	[
		'<input type="checkbox" value="1">',
		'<label><input type="checkbox" value="2"> <"></label>',
	],
	'anonymous checkboxes'
);

is_deeply(
	[ $form->check(undef, [{}, undef], 1) ],
	[
		'<input checked type="checkbox" value="1">',
		'<input checked type="checkbox" value="2">',
	],
	'multiple checked checkboxes'
);

{
	local $, = ' | ';
	is(
		scalar $form->check(foo => [1, 0], {value => 0}),
		join(' | ',
			'<label><input id="foo_1" name="foo" type="checkbox" value="1"> 1</label>',
			'<label><input id="foo_2" name="foo" type="checkbox" value="2"> 0</label>',
		),
		'merged checkboxes'
	);
}

is_deeply(
	[ $form->check(undef, [{value => 4, type => 'radio'}], [1, 0, 0], {value => 3}) ],
	[
		'<input checked type="radio" value="4">',
		'<input type="checkbox" value="2">',
		'<input checked type="checkbox" value="3">',
	],
	'various checkboxes'
);

# defaults

my $defform = HTML::Form::Simple->new({foo => '<">', '' => 'empty', 0 => 0});
ok($defform, 'form with defaults');

is(
	$defform->hidden(''),
	'<input name="" type="hidden" value="empty">',
	'hidden with default'
);

is(
	$defform->hidden(undef),
	'<input type="hidden">',
	'nameless hidden'
);

is(
	$defform->text('foo'),
	'<input id="foo" name="foo" type="text" value="&lt;&quot;>">',
	'input with default'
);

is(
	$defform->text('foo', {value => 'custom'}),
	'<input id="foo" name="foo" type="text" value="custom">',
	'input with value and default'
);

is_deeply(
	[ $defform->radio(0 => [1, 0]) ],
	[
		'<input id="0_1" name="0" type="radio" value="1">',
		'<input checked id="0_0" name="0" type="radio" value="0">',
	],
	'select with default'
);

#TODO

